/* $Id: clientconnectionlistener.cpp,v 1.2 2005/06/28 13:55:18 chfreund Exp $ */

#include "clientconnectionlistener.hpp"
#include "global.hpp"
#include "servercommunicator.hpp"
#include "clientconnectiontcp.hpp"
#include "clientconnectiontcpudp.hpp"
#include "tcpconnection.hpp"
#include "wopsettings.hpp"

ClientConnectionListenerTCP::ClientConnectionListenerTCP
( ServerCommunicator* communicator, Uint16 port )
	: m_communicator( communicator ), m_port( port ) {
	LOG( 1 ) INFO( "ClientConnectionListenerTCP::ClientConnectionListenerTCP(): initializing\n" );
	
	// Resolve the listening address
	IPaddress listenAddress;
	SDLNet_ResolveHost( &listenAddress, 0, m_port );
	
	// Create the TCP socket
	m_socket = SDLNet_TCP_Open( &listenAddress );
	ASSERT( m_socket, "ClientConnectionListenerTCP::ClientConnectionListenerTCP: could not create game socket (%s)\n", SDLNet_GetError() );
	
	// Create socket set
	m_socketSet = SDLNet_AllocSocketSet( 1 );
	SDLNet_TCP_AddSocket( m_socketSet, m_socket );
	
	m_baitSocket = SDLNet_UDP_Open( 0xb8b8 );
	m_baitSocketSet = SDLNet_AllocSocketSet( 1 );
	SDLNet_UDP_AddSocket( m_baitSocketSet, m_baitSocket );
	m_baitPacket = SDLNet_AllocPacket( 200 );
}

ClientConnectionListenerTCP::~ClientConnectionListenerTCP() {
	LOG( 1 ) INFO( "ClientConnectionListenerTCP::~ClientConnectionListenerTCP: destroying\n" );
	
	// cleanup
	if ( m_socket ) {
		SDLNet_TCP_Close( m_socket );
		SDLNet_TCP_DelSocket( m_socketSet, m_socket );
	}
	if ( m_socketSet ) {
		SDLNet_FreeSocketSet( m_socketSet );
	}
	
	if ( m_baitSocket ) {
		SDLNet_UDP_Close( m_baitSocket );
		SDLNet_UDP_DelSocket( m_baitSocketSet, m_baitSocket );
	}
	if ( m_baitSocketSet ) {
		SDLNet_FreeSocketSet( m_baitSocketSet );
	}
	if ( m_baitPacket ) {
		SDLNet_FreePacket( m_baitPacket );
	}
}

ClientConnection* ClientConnectionListenerTCP::getNewConnection( Uint8 clientID ) {
	// check for activity on socket set
	if ( SDLNet_CheckSockets( m_socketSet, 0 ) ) {
		LOG( 2 ) INFO( "ClientConnectionListenerTCP::getNewConnection: accepting connection\n" );
		TCPsocket socket = SDLNet_TCP_Accept( m_socket );
		ClientConnection* connection = 0;
		// try a TCP/UDP connection
		ClientConnectionTCPUDP* tcpudpConnection = new ClientConnectionTCPUDP( socket );
		tcpudpConnection->openConnection( clientID );
		if ( !tcpudpConnection->checkUdpConnection() ) {
			ClientConnectionTCP* tcpConnection = tcpudpConnection->createClientConnectionTCP();
			delete tcpudpConnection;
			connection = tcpConnection;
		}
		else
			connection = tcpudpConnection;
		return connection;
	}
	
	handleBaitPackets();
	return 0;
}

void ClientConnectionListenerTCP::handleBaitPackets() {
	if ( SDLNet_CheckSockets( m_baitSocketSet, 0 ) > 0 ) {
		if ( SDLNet_UDP_Recv( m_baitSocket, m_baitPacket ) == 1 ) {
			LOG( 2 ) INFO( "ClientConnectionListenerTCP::handleBaitPackets: Received bait packet (len: %i) from %i.%i.%i.%i:%i\n",
				m_baitPacket->len, DECODE_IP( m_baitPacket->address.host),
				m_baitPacket->address.port);
			Message headerMessage;
			Uint8* bufferPointer = m_baitPacket->data;
			headerMessage.deserialize( bufferPointer );
			if ( headerMessage.type == Message::BAIT ) {
				BaitMessage baitMessage( headerMessage );
				baitMessage.deserializeData( bufferPointer );
				if ( baitMessage.magicData == 0xb8b8 ) {
					LOG( 2 ) INFO( "ClientConnectionListenerTCP::handleBaitPackets: valid client request\n" );
					BaitAnswerMessage answerMessage;
					answerMessage.serverPort = m_port;
					answerMessage.theme = WopSettings::getInstance()->getTheme();
					answerMessage.numberPlayers = m_communicator->getWorld()->getNumberPlayers();
					bufferPointer = m_baitPacket->data;
					answerMessage.serialize( bufferPointer );
					m_baitPacket->len = answerMessage.getSerializeBufferSize();
					if ( CHECK( SDLNet_UDP_Send( m_baitSocket, -1, m_baitPacket ) == 1,
									"ClientConnectionListenerTCP::handleBaitPackets: Could not send bait packet (%s)\n",
									SDLNet_GetError() ) ) {
					}
					else {
						LOG( 2 ) INFO( "ClientConnectionListenerTCP::handleBaitPackets: invalid client request (length = %d)\n",
											m_baitPacket->len );
					}
				}
			}
		}
	}
}
